/**
 * @file simple-toasts.js
 *
 * Replaces the standard drupal status messages and creates javascript toasts
 */
(function ($, Drupal, once, drupalSettings) {

  'use strict';

  Drupal.simpleToasts = {
    id: 'drupal-simple-toasts',
    context: 'body',
    bootstrapVersion: null,
    defaultDuration: 10000,
    animation: {
      in: 'fade-in',
      out: 'fade-out',
      duration: 300,
    },
    messageTypeDuration: {
      status: null,
      success: null,
      warning: null,
    },
    position: 'top-right', /** top-left/top-right/top-center/bottom-left/bottom-right/bottom-center - Where the toast will show up **/
    iconStatus: '<svg xmlns="http://www.w3.org/2000/svg" width="20" height="20" fill="currentColor" viewBox="0 0 16 16"><path d="M2.5 8a5.5 5.5 0 0 1 8.25-4.764.5.5 0 0 0 .5-.866A6.5 6.5 0 1 0 14.5 8a.5.5 0 0 0-1 0 5.5 5.5 0 1 1-11 0"/><path d="M15.354 3.354a.5.5 0 0 0-.708-.708L8 9.293 5.354 6.646a.5.5 0 1 0-.708.708l3 3a.5.5 0 0 0 .708 0z"/></svg>',
    iconWarning: '<svg xmlns="http://www.w3.org/2000/svg" width="20" height="20" fill="currentColor" viewBox="0 0 16 16"><path d="M7.938 2.016A.13.13 0 0 1 8.002 2a.13.13 0 0 1 .063.016.15.15 0 0 1 .054.057l6.857 11.667c.036.06.035.124.002.183a.2.2 0 0 1-.054.06.1.1 0 0 1-.066.017H1.146a.1.1 0 0 1-.066-.017.2.2 0 0 1-.054-.06.18.18 0 0 1 .002-.183L7.884 2.073a.15.15 0 0 1 .054-.057m1.044-.45a1.13 1.13 0 0 0-1.96 0L.165 13.233c-.457.778.091 1.767.98 1.767h13.713c.889 0 1.438-.99.98-1.767z"/><path d="M7.002 12a1 1 0 1 1 2 0 1 1 0 0 1-2 0M7.1 5.995a.905.905 0 1 1 1.8 0l-.35 3.507a.552.552 0 0 1-1.1 0z"/></svg>',
    iconError: '<svg xmlns="http://www.w3.org/2000/svg" width="20" height="20" fill="currentColor" viewBox="0 0 16 16"><path d="M4.54.146A.5.5 0 0 1 4.893 0h6.214a.5.5 0 0 1 .353.146l4.394 4.394a.5.5 0 0 1 .146.353v6.214a.5.5 0 0 1-.146.353l-4.394 4.394a.5.5 0 0 1-.353.146H4.893a.5.5 0 0 1-.353-.146L.146 11.46A.5.5 0 0 1 0 11.107V4.893a.5.5 0 0 1 .146-.353zM5.1 1 1 5.1v5.8L5.1 15h5.8l4.1-4.1V5.1L10.9 1z"/><path d="M7.002 11a1 1 0 1 1 2 0 1 1 0 0 1-2 0M7.1 4.995a.905.905 0 1 1 1.8 0l-.35 3.507a.552.552 0 0 1-1.1 0z"/></svg>',
    messages: [],
    count: 0,
    totalRemovingHeight: 0,
    drupalMessages: new Drupal.Message(),
    init: function (messages) {
      this.messages = messages || [];
      if(this.messages.length) {
        this.drupalMessages.clear();
        this.getSettings();
        this.createMessages(messages);
      }
    },
    getSettings: function () {
      if (typeof drupalSettings.simple_toasts !== 'undefined') {
        if (typeof drupalSettings.simple_toasts.status_duration !== 'undefined') {
          this.messageTypeDuration.status = drupalSettings.simple_toasts.status_duration;
        } else {
          this.messageTypeDuration.status = this.defaultDuration;
        }
        if (typeof drupalSettings.simple_toasts.warning_duration !== 'undefined') {
          this.messageTypeDuration.warning = drupalSettings.simple_toasts.warning_duration;
        } else {
          this.messageTypeDuration.warning = this.defaultDuration;
        }
        if (typeof drupalSettings.simple_toasts.error_duration !== 'undefined') {
          this.messageTypeDuration.error = drupalSettings.simple_toasts.error_duration;
        } else {
          this.messageTypeDuration.error = this.defaultDuration;
        }
        if (typeof drupalSettings.simple_toasts.position !== 'undefined') {
          this.position = drupalSettings.simple_toasts.position;
        }
        if (typeof drupalSettings.simple_toasts.animation_in !== 'undefined') {
          this.animation.in = drupalSettings.simple_toasts.animation_in;
        }
        if (typeof drupalSettings.simple_toasts.animation_out !== 'undefined') {
          this.animation.out = drupalSettings.simple_toasts.animation_out;
        }
      }
    },
    createMessages: function () {
      if (Array.isArray(this.messages) && this.messages.length) {
        for (let i in this.messages) {
          switch (this.messages[i].type) {
            case 'status':
            default:
              this.messages[i].type = 'status';
              this.messages[i].icon = this.iconStatus;
              this.messages[i].title = 'Status message';
              break;
            case 'warning':
              this.messages[i].type = 'warning';
              this.messages[i].icon = this.iconWarning;
              this.messages[i].title = 'Warning message';
              break;
            case 'error':
              this.messages[i].type = 'error';
              this.messages[i].icon = this.iconError;
              this.messages[i].title = 'Error message';
              break;
          }
        }
        this.render();
      }
    },
    render: function () {
      let toastContainer = document.querySelector('#st-toast-container');
      if (!toastContainer) {
        toastContainer = document.createElement('div');
        toastContainer.classList.add('st-toast-container', this.position);
        toastContainer.id = 'st-toast-container';
        toastContainer.setAttribute('role', 'alert');
        toastContainer.setAttribute('aria-live', 'polite');
        toastContainer.setAttribute('aria-atomic', 'true');
        $('body').prepend(toastContainer);
      }
      this.messages.forEach((message, index) => {
        const toastElement = this.renderItem(message, index);
        toastContainer.appendChild(toastElement);
      });
    },
    renderItem: function (message) {
      let toast = document.createElement('div');
      toast.classList.add('st-toast', 'st-' + message.type);
      toast.id = 'st-toast-' + this.count;
      toast.setAttribute('role', 'alert');
      toast.setAttribute('aria-live', 'assertive');
      toast.setAttribute('aria-atomic', 'true');

      //Animation
      if (this.animation.in !== 'none' || this.animation.out !== 'none') {
        toast.classList.add('animate');
        if (this.animation.in !== 'none') {
          toast.classList.add('animate-in', this.animation.in);
        }
        if (this.animation.out !== 'none') {
          toast.classList.add('animate-out', this.animation.out);
        }
        if (this.messages.length > 1) {
          toast.classList.add('animate-in--' + this.count);
        }
      }
      toast.classList.add('st-show');
      // Handle the auto-hide functionality
      if (this.messageTypeDuration[message.type] === 0) {
        // Disable auto-hide if duration is set to 0
        toast.dataset.stAutohide = 'false';
      } else {
        toast.dataset.stAutohide = 'true';
        toast.dataset.stDelay = this.messageTypeDuration[message.type];
      }

      // Create the toast header
      let toastHeader = document.createElement('div');
      toastHeader.classList.add('st-toast-header', 'st-bg-' + message.type);
      toastHeader.innerHTML = '<div class="visually-hidden">' + message.title + '</div>';

      // Create the toast body
      let toastBody = document.createElement('div');
      toastBody.classList.add('st-toast-body');
      toastBody.innerHTML = '' +
        '<span class="st-toast-icon">' + message.icon + '</span>' +
        '<span class="st-toast-message">' + message.message + '</span>' +
        '<button type="button" class="st-toast-close" aria-label="Close"><svg width="9" height="9" viewBox="0 0 44 44" aria-hidden="true" focusable="false"><path d="M0.549989 4.44999L4.44999 0.549988L43.45 39.55L39.55 43.45L0.549989 4.44999Z" /><path d="M39.55 0.549988L43.45 4.44999L4.44999 43.45L0.549988 39.55L39.55 0.549988Z" /></svg></button>';

      //Assemble toast
      toast.appendChild(toastHeader);
      toast.appendChild(toastBody);

      // Add close button functionality
      toast.querySelector('.st-toast-close').addEventListener('click', () => {
        Drupal.simpleToasts.hideToast(toast);
      });

      // Auto-remove toast after delay if applicable
      if (toast.dataset.stAutohide === 'true') {
        let autoRemoveTimeout;
        // Function to start or restart the timeout
        const startTimeout = () => {
          autoRemoveTimeout = setTimeout(() => {
            Drupal.simpleToasts.hideToast(toast);
          }, parseInt(toast.dataset.stDelay, 10));
        };
        // Start the initial timeout
        startTimeout();
        // Clear the timeout on mouse enter
        toast.addEventListener('mouseenter', () => clearTimeout(autoRemoveTimeout));
        // Reapply the timeout on mouse leave
        toast.addEventListener('mouseleave', startTimeout);
      }
      this.count++;

      return toast;
    },
    hideToast(toast) {
      toast.classList.add('st-hide');
      toast.classList.remove('st-show');
      // Listen for the animation end event
      toast.addEventListener('animationend', () => {
        //this.onToastHide(toast);
        toast.remove();
      });
    },
    /** Callback function for when a toast is hidden,
     * Removes the toast from the DOM and adjusts the positions of the remaining toasts
     * Handles the positions of the remaining toasts using jQuery animation
     * @TODO not used at the moment, animation on multiple bottom aligned posts needs improvement
     +*/
    onToastHide(toast) {
      if (!toast.classList.contains('animate-out')) {
        toast.remove();
      } else {
        // Get the container as jQuery & its dimensions
        const $container = $(toast.parentNode);
        const containerHeight = $container.height();
        const containerOuterHeight = $container.outerHeight(true);

        //Get the toast as jQuery & its dimensions
        const $toast = $(toast);
        const toastOuterHeight = $toast.outerHeight(true);
        const toastHeight = $toast.height();
        const toastMarginBottom = $toast.css('margin-bottom');

        this.totalRemovingHeight += toastOuterHeight;

        let $next = null;
        let $prev = null;
        if (this.position.startsWith('top')) {
          $next = $toast.next();
        } else if (this.position.startsWith('bottom')) {
          $prev = $toast.prev();
        }
        if (($next && $next.length) || ($prev && $prev.length)) {
          $container.height(containerHeight);
        }

        toast.remove();
        //Top positioned toasts
        if ($next && $next.length) {
          $next.css({ 'margin-top': toastHeight });
          $next.animate({ 'margin-top': 0 }, this.animation.duration, 'swing', function () {
            $container.css('height', '');
            //Not strictly required here but resetting the totalRemovingHeight
            Drupal.simpleToasts.totalRemovingHeight = 0;
          });
        }
        //Bottom positioned toasts
        else if ($prev && $prev.length) {
          const removingCount = $('.toast-container .toast.removing').length;
          $prev.css({ 'margin-bottom': toastHeight });
          $prev.animate({ 'margin-bottom': toastMarginBottom }, this.animation.duration, 'swing', function () {
            if (removingCount === 0) {
              $container.animate({ height: (containerOuterHeight - Drupal.simpleToasts.totalRemovingHeight) }, Drupal.simpleToasts.animation.duration, 'swing', function () {
                $container.css('height', '');
                Drupal.simpleToasts.totalRemovingHeight = 0;
              });
            }
          });
        }

      }
      this.count--;
    },
  };

  /**
   * Registers behaviours
   */
  // Drupal.behaviors.simpleToasts = {
  //   attach: function (context, settings) {
  //     once(Drupal.simpleToasts.id, Drupal.simpleToasts.context, context).forEach(function (element) {
  //       console.log('simpleToasts behaviour attached');
  //       Drupal.simpleToasts.init();
  //       if (typeof settings.simple_toasts !== 'undefined' && typeof settings.simple_toasts.messages_array !== 'undefined') {
  //         Drupal.simpleToasts.createMessages(settings.simple_toasts.messages_array);
  //         settings.simple_toasts.messages_array = null;
  //       }
  //     });
  //   },
  // };

  /**
   * Create bootstrap toast messages on ajax requests
   */
  Drupal.AjaxCommands.prototype.simpleToastsCommand = function (ajax, response, status) {
    Drupal.simpleToasts.init(response.messages_array);
  };

}(jQuery, Drupal, once, drupalSettings));
